pub struct ID {
pub client: u64,
pub clock: u32,
}Expand description
Block identifier, which allows to uniquely identify any element insertion in a global scope (across different replicas of the same document). It consists of client ID (which is a unique document replica identifier) and monotonically incrementing clock value.
ID corresponds to a Lamport timestamp in terms of its properties and guarantees.
Fields
client: u64Unique identifier of a client.
clock: u32Monotonically incrementing sequence number, which informs about order of inserted item
operation in a scope of a given client. This value doesn’t have to increase by 1, but
instead is increased by number of countable elements which make a content of an inserted
block.
Implementations