Struct XmlElementPrelim

pub struct XmlElementPrelim<I, T>(/* private fields */)
where
    I: IntoIterator<Item = T>,
    T: XmlPrelim;
A preliminary type that will be materialized into an XmlElementRef once it will be integrated into Yrs document.

Implementations

Source

impl<I, T> XmlElementPrelim<I, T>
where I: IntoIterator<Item = T>, T: XmlPrelim,

pub fn new<S>(tag: S, iter: I) -> XmlElementPrelim<I, T>
where S: Into<Arc<str>>,

Source

impl XmlElementPrelim<Option<XmlTextPrelim<String>>, XmlTextPrelim<String>>