Struct XmlTextPrelim

pub struct XmlTextPrelim<T>(/* private fields */)
where
    T: Borrow<str>;
A preliminary type that will be materialized into an XmlTextRef once it will be integrated into Yrs document.

Implementations

Source

impl<T> XmlTextPrelim<T>
where T: Borrow<str>,

pub fn new(str: T) -> XmlTextPrelim<T>