pub enum Any {
Null,
Undefined,
Bool(bool),
Number(f64),
BigInt(i64),
String(Arc<str>),
Buffer(Arc<[u8]>),
Array(Arc<[Any]>),
Map(Arc<HashMap<String, Any>>),
}Expand description
Any is an enum with a potentially associated value that is used to represent JSON values and supports efficient encoding of those values.
Variants
Null
Undefined
Bool(bool)
Number(f64)
BigInt(i64)
String(Arc<str>)
Buffer(Arc<[u8]>)
Array(Arc<[Any]>)
Map(Arc<HashMap<String, Any>>)
Implementations