Value

Enum Value 

pub enum Value {
    Any(Any),
    YText(TextRef),
    YArray(ArrayRef),
    YMap(MapRef),
    YXmlElement(XmlElementRef),
    YXmlFragment(XmlFragmentRef),
    YXmlText(XmlTextRef),
    YDoc(Doc),
    UndefinedRef(BranchPtr),
}
Value that can be returned by Yrs data types. This includes Any which is an extension representation of JSON, but also nested complex collaborative structures specific to Yrs.

Variants



Any(Any)

Any value that it treated as a single element in it’s entirety.



YText(TextRef)

Instance of a TextRef.



YArray(ArrayRef)

Instance of an ArrayRef.



YMap(MapRef)

Instance of a MapRef.



YXmlElement(XmlElementRef)

Instance of a XmlElementRef.



YXmlFragment(XmlFragmentRef)

Instance of a XmlFragmentRef.



YXmlText(XmlTextRef)

Instance of a XmlTextRef.



YDoc(Doc)

Subdocument.



UndefinedRef(BranchPtr)

Instance of a shared collection of undefined type. Usually happens when it refers to a root type that has not been defined locally. Can also refer to a [WeakRef] if “weak” feature flag was not set.

Implementations

Source

impl Value

pub fn cast<T>(self) -> Result<T, Value>
where T: TryFrom<Value, Error = Value>,

pub fn to_string<T>(self, txn: &T) -> String
where T: ReadTxn,

Converts current value into stringified representation.

pub fn try_branch(&self) -> Option<&Branch>