pub struct MapRef(/* private fields */);Expand description
Collection used to store key-value entries in an unordered manner. Keys are always represented as UTF-8 strings. Values can be any value type supported by Yrs: JSON-like primitives as well as shared data types.
In terms of conflict resolution, MapRef uses logical last-write-wins principle, meaning the past updates are automatically overridden and discarded by newer ones, while concurrent updates made by different peers are resolved into a single value using document id seniority to establish order.
Example
use yrs::{any, Doc, Map, MapPrelim, Transact};
use yrs::types::ToJson;
let doc = Doc::new();
let map = doc.get_or_insert_map("map");
let mut txn = doc.transact_mut();
// insert value
map.insert(&mut txn, "key1", "value1");
// insert nested shared type
let nested = map.insert(&mut txn, "key2", MapPrelim::from([("inner", "value2")]));
nested.insert(&mut txn, "inner2", 100);
assert_eq!(map.to_json(&txn), any!({
"key1": "value1",
"key2": {
"inner": "value2",
"inner2": 100
}
}));
// get value
assert_eq!(map.get(&txn, "key1"), Some("value1".into()));
// remove entry
map.remove(&mut txn, "key1");
assert_eq!(map.get(&txn, "key1"), None);Trait Implementations
Sourceimpl DeepObservable for MapRef
impl DeepObservable for MapRef
Sourcefn observe_deep<F>(&self, f: F) -> Subscription
fn observe_deep<F>(&self, f: F) -> Subscription
impl Eq for MapRef
Sourceimpl Into<MapRef> for XmlElementRef
impl Into<MapRef> for XmlElementRef
Sourceimpl Into<MapRef> for XmlHookRef
impl Into<MapRef> for XmlHookRef
Sourceimpl Map for MapRef
impl Map for MapRef
Sourcefn len<T>(&self, _txn: &T) -> u32where
T: ReadTxn,
fn len<T>(&self, _txn: &T) -> u32where
T: ReadTxn,
Returns a number of entries stored within current map.
Sourcefn keys<'a, T>(&'a self, txn: &'a T) -> Keys<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
fn keys<'a, T>(&'a self, txn: &'a T) -> Keys<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
Returns an iterator that enables to traverse over all keys of entries stored within
current map. These keys are not ordered.
Sourcefn values<'a, T>(&'a self, txn: &'a T) -> Values<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
fn values<'a, T>(&'a self, txn: &'a T) -> Values<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
Returns an iterator that enables to traverse over all values stored within current map.
Sourcefn iter<'a, T>(&'a self, txn: &'a T) -> MapIter<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
fn iter<'a, T>(&'a self, txn: &'a T) -> MapIter<'a, &'a T, T> ⓘwhere
T: ReadTxn + 'a,
Returns an iterator that enables to traverse over all entries - tuple of key-value pairs -
stored within current map.
Sourcefn insert<K, V>(
&self,
txn: &mut TransactionMut<'_>,
key: K,
value: V,
) -> <V as Prelim>::Return
fn insert<K, V>( &self, txn: &mut TransactionMut<'_>, key: K, value: V, ) -> <V as Prelim>::Return
Inserts a new
value under given key into current map. Returns an integrated value.Sourcefn remove(&self, txn: &mut TransactionMut<'_>, key: &str) -> Option<Value>
fn remove(&self, txn: &mut TransactionMut<'_>, key: &str) -> Option<Value>
Removes a stored within current map under a given
key. Returns that value or None if
no entry with a given key was present in current map. Read moreSourcefn get<T>(&self, txn: &T, key: &str) -> Option<Value>where
T: ReadTxn,
fn get<T>(&self, txn: &T, key: &str) -> Option<Value>where
T: ReadTxn,
Returns a value stored under a given
key within current map, or None if no entry
with such key existed.Sourcefn contains_key<T>(&self, _txn: &T, key: &str) -> boolwhere
T: ReadTxn,
fn contains_key<T>(&self, _txn: &T, key: &str) -> boolwhere
T: ReadTxn,
Checks if an entry with given
key can be found within current map.Sourcefn clear(&self, txn: &mut TransactionMut<'_>)
fn clear(&self, txn: &mut TransactionMut<'_>)
Clears the contents of current map, effectively removing all of its entries.
Sourceimpl Observable for MapRef
impl Observable for MapRef
type Event = MapEvent
Sourcefn observe<F>(&self, f: F) -> Subscription
fn observe<F>(&self, f: F) -> Subscription
Subscribes a given callback to be triggered whenever current y-type is changed.
A callback is triggered whenever a transaction gets committed. This function does not
trigger if changes have been observed by nested shared collections. Read more