ObserverMut

Struct ObserverMut 

pub struct ObserverMut<E> { /* private fields */ }
Data structure used to handle publish/subscribe callbacks of specific type. Observers perform subscriber changes in thread-safe manner, using atomic hardware intrinsics.

Implementations

Source

impl<E> ObserverMut<E>

pub fn new() -> ObserverMut<E>

Creates a new [ObserverMut with no active callbacks.

pub fn clean(&self)

Cleanup already released subscriptions. Whenever a Subscription is dropped, the callback is released. However, the weak reference to callback may still be kept around until it becomes touched by operations such as Observer::subscribe or Observer::callbacks.

This method allows to perform stale callback cleanup without waiting for callbacks to be visited.

pub fn subscribe<F>(&self, callback: F) -> Subscription
where F: Fn(&mut TransactionMut<'_>, &mut E) + 'static,

Subscribes a callback parameter to a current Observer. Returns a subscription object which - when dropped - will unsubscribe current callback.

pub fn callbacks(&self) -> Option<CallbacksMut<'_, E>>

Returns a snapshot of callbacks subscribed to this observer at the moment when this method has been called. This snapshot can be iterated over to get access to individual callbacks and trigger them.