pub struct Subscription { /* private fields */ }Expand description
Subscription handle returned by Observer::subscribe methods, which will unsubscribe corresponding callback when dropped.
If implicit callback unsubscribe on drop is undesired, this structure can be cast into [SubscriptionId] which is an identifier of the same subscription, which in turn must be used manually via [Observer::unsubscribe] to perform usubscribe.