pub struct XmlTextPrelim<T>(/* private fields */)
where
T: Borrow<str>;Expand description
A preliminary type that will be materialized into an XmlTextRef once it will be integrated into Yrs document.
Implementations
Sourceimpl<T> XmlTextPrelim<T>
impl<T> XmlTextPrelim<T>
pub fn new(str: T) -> XmlTextPrelim<T>