pub struct Update { /* private fields */ }Expand description
Update type which contains an information about all decoded blocks which are incoming from a remote peer. Since these blocks are not yet integrated into current document’s block store, they still may require repairing before doing so as they don’t contain full data about their relations.
Update is conceptually similar to a block store itself, however the work patters are different.
Implementations
Sourceimpl Update
impl Update
pub fn new() -> Update
Sourcepub fn state_vector(&self) -> StateVector
pub fn state_vector(&self) -> StateVector
Returns a state vector representing an upper bound of client clocks included by blocks stored in current update.
Sourcepub fn merge(&mut self, other: Update)
pub fn merge(&mut self, other: Update)
Merges another update into current one. Their blocks are deduplicated and reordered.
pub fn merge_updates<T>(block_stores: T) -> Updatewhere
T: IntoIterator<Item = Update>,